@@ -1,35 +1,33 @@
 #
-# $Id: Engine.pm,v 1.9 1999/03/19 03:38:05 jsmith Exp $
+# $Id: Engine.pm,v 1.34 1999/05/15 15:06:31 jsmith Exp $
 #
 # Author: James G. Smith
 #
 # Copyright (C) 1999
 #
 # This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.
+# under the terms of the Artistic Licence.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
-# more details.
+# FITNESS FOR A PARTICULAR PURPOSE. See the Artistic License for more details.
 #
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 675 Mass Ave, Cambridge, MA 02139, USA.
-#
-# The author may be reached at <j-smith@physics.tamu.edu>
+# The author may be reached at <jsmith@nostrum.com>
 #
 
 package CGI::WeT::Engine;
 
 use strict;
 use Carp;
-use vars qw($VERSION);
+use vars qw($VERSION @ISA);
 use integer;
+use Apache::Constants;
+use IO::File ();
+# use CGI qw/:standard/;
 
-$VERSION = '0.6.3';
+# @ISA = (qw(CGI));
+
+( $VERSION ) = '$Revision: 1.34 $ ' =~ /\$Revision:\s+([^\s]+)/;
 
 =pod
 
@@ -45,7 +43,7 @@ CGI::WeT::Engine - Theme engine for the CGI::WeT package
 
 This module provides a web site with the ability to provide themes.  It is
 designed to work seamlessly with B<mod_perl> but can be made to work without
-B<mod_perl> without too much difficulty.  
+B<mod_perl> without too much difficulty.
 
 =head1 NOT USING MOD_PERL
 
@@ -76,7 +74,7 @@ all themed page requests.  A sample script might be
         while(<IN>) {
 
             last if /^\s*$/;
-            next if /^\s*#/;
+            next if /^\s*\#/;
 
             chomp;
 
@@ -93,11 +91,11 @@ all themed page requests.  A sample script might be
         # slurp up the rest of the file
         #
 
-        $engine->body_push((<IN>));
+        $engine->print((<IN>));
         close IN;
     }
     
-    $r->print( $engine->render_page );
+    # page is rendered when $engine is destroyed...
 
 Apache must then be configured to call the CGI script for all files that
 are themed.
@@ -135,6 +133,9 @@ Returns a reference to a new rendering engine.  The returned object will
 have parsed the arguments to the URL (if GET or POST).  The returned object
 will need to be set up before a page can be rendered.
 
+When the object returned by B<new> is destroyed, the page is rendered to
+STDOUT.
+
 =cut
 
 sub new {
@@ -154,6 +155,18 @@ sub new {
     if($self->{'MOD_PERL'}) {
 	$r = Apache->request;
 
+#
+# take care of Apache::Filter detection here...
+#
+	if(defined &Apache::filter_input) {
+	    no strict 'subs';
+	    $self->{'FILTERED'} = $Apache::Filter::VERSION || 1;
+	    $self->{'STDOUT'} = \STDOUT;
+	} else {
+	    $self->{'FILTERED'} = 0;
+	    $self->{'STDOUT'} = $r;
+	}
+
 	if($r->method eq 'GET') {
 	    $in = $r->args;
 	} elsif($r->method eq 'POST') {
@@ -181,7 +194,7 @@ sub new {
 	
 	$key =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
 	$val =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
-	$val =~ s/[\r\n]+/ /g;
+	$val =~ s/\r+//g;
 	if(defined $in{$key}) {
 	    my $collect = $in{$key};
 	    if(ref $collect) {
@@ -203,7 +216,7 @@ sub new {
 	    
 	    $key =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
 	    $val =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
-	    $val =~ s/[\r\n]+/ /g;
+	    $val =~ s/\r+//g;
 	    if(defined $cookiein{$key}) {
 		my $collect = $cookiein{$key};
 		if(ref $collect) {
@@ -222,7 +235,6 @@ sub new {
     
     $self->arguments_push(\%in);
 
-    if($self->{'MOD_PERL'}) {
 =pod
 
 If using mod_perl, the following variables may be set using B<PerlSetVar>:
@@ -235,6 +247,8 @@ If using mod_perl, the following variables may be set using B<PerlSetVar>:
    WeT_Email - email of the administrator
    WeT_ProblemsEmail - email for bug reports and other problems
    WeT_DefaultTheme - initial theme people will see
+                      this can be overridden with the environment variable
+                      `WET_THEME'
    WeT_UseSSLURLs - whether to add :SSL and :NOSSL to the
                     end of URLs generated by the engine or not. (This
                     is considered true if defined.)  This is still
@@ -257,8 +271,11 @@ to be defined:
    $engine->{'SSL_URLS'} -- corresponds to WeT_UseSSLURLs
    $engine->{'AC'} -- corresponds tp WeT_AnonymousCoward
 =cut
+# ' for Emacs
+# ` for Emacs
 
-	# get config from httpd.conf
+    if($self->{'MOD_PERL'}) {
+        # get config from httpd.conf
 	$self->{'SITENAME'} = $r->dir_config('WeT_SiteName');
 	$self->{'URLBASES'}->{'URLBASE'} = 
 	    $r->dir_config('WeT_SiteRoot') || '/';
@@ -269,8 +286,9 @@ to be defined:
 	$self->{'EMAIL'} = $r->dir_config('WeT_Email') || $ENV{'SERVER_ADMIN'};
 	$self->{'PROBLEMS_EMAIL'} = 
 	    $r->dir_config('WeT_ProblemsEmail') || $self->{'EMAIL'};
-	$self->{'DEFAULT_THEME'} = $r->dir_config('WeT_DefaultTheme') ||
-            'plain';
+	$self->{'DEFAULT_THEME'} = $ENV{'WET_THEME'} || 
+	    $r->dir_config('WeT_DefaultTheme') ||
+		'plain';
         $self->{'SSL_URLS'} = $r->dir_config('WeT_UseSSLURLs');
         $self->{'AC'} = $r->dir_config('WeT_AnonymousCoward');
         foreach (map((m/(.*)::$/),
@@ -291,7 +309,7 @@ to be defined:
             $self->{'DOCUMENTROOT'} = $ENV{'DOCUMENT_ROOT'};
             $self->{'EMAIL'} = $ENV{'SERVER_ADMIN'};
             $self->{'PROBLEMS_EMAIL'} = $self->{'EMAIL'};
-            $self->{'DEFAULT_THEME'} = 'plain';
+            $self->{'DEFAULT_THEME'} = $ENV{'WET_THEME'} || 'plain';
             $self->{'AC'} = 'Anonymous Coward';
         }
     }
@@ -307,20 +325,76 @@ to be defined:
     return $self;
 }
 
+sub DESTROY {
+    my $engine = shift;
+    my $r;
+
+    return if $engine->{'INTERNAL_USE_ONLY'};
+
+    if($engine->{'MOD_PERL'} ) {
+	$r = Apache->request;
+	my $fh = $engine->{'STDOUT'};
+
+	$r->content_type('text/html');
+
+	$r->send_http_header unless $engine->{'FILTERED'};
+
+	$fh->print($engine->render_page);
+    } else {
+	print "Content-type: text/html\n\n";
+	print $engine->render_page;
+    }
+}
+
+=pod
+
+=item $engine->internal_use_only
+
+This function will mark the $engine object as being for internal use only
+(hence its name).  That is, it will not output a page to output upon
+destruction.  Returns the previous value.  If an argument is present, the
+internal flag is set to that value.
+
+This function is valuable if a CGI script decides it needs to redirect instead
+of outputing HTML.  Calling this function will disable automatic HTML output.
+
+=cut
+
+sub internal_use_only {
+    my $engine = shift;
+    my $v = $engine->{'INTERNAL_USE_ONLY'};
+
+    if(scalar(@_)) {
+	$engine->{'INTERNAL_USE_ONLY'} = shift;
+    } else {
+	$engine->{'INTERNAL_USE_ONLY'} = 1;
+    }
+
+    return $v;
+}
+
 =pod
 
 =item $engine->content_pop
 
 This function returns the item on the top of the content
 stack.  Used primarily in the rendering code and the B<CGI::WeT::Modules>
-extensions to the engine.
+extensions to the engine.  An optional argument specifies how many elements
+to pop.
 
 =cut
 
 sub content_pop {
     my $self = shift;
+    my $num = shift || 1;
 
-    return pop @{ $$self{CONTENT} };
+    $num = 
+	($num > scalar(@{ $$self{'CONTENT'} })) ? 
+	    scalar(@{ $$self{'CONTENT'} }) : 
+		$num;
+
+
+    return splice(@{ $$self{'CONTENT'} }, -$num);
 }
 
 =pod
@@ -352,7 +426,7 @@ extensions to the engine.
 sub content_peek {
     my $self = shift;
 
-    return ${ $$self{CONTENT} }[$#{ $$self{CONTENT} }];
+    return ${ $$self{CONTENT} }[-1];
 }
 
 =pod
@@ -452,6 +526,8 @@ B<Author>, B<Keywords>, B<Date> - These three are placed verbatim in META
 tags in the header.  Useful information for search engines.
 
 =cut
+# ' for Emacs
+# ` for Emacs
 
 sub headers_push {
     my $self = shift;
@@ -509,25 +585,14 @@ sub header {
 
 =pod
 
-=item $engine->body_push(B<array>)
-
 =item $engine->print(B<array>)
 
 This function places B<array> at the end of the body content caching it for
 later use by the rendering code.  Text placed in the body cannot be
 removed by a provided method.
 
-Use of B<body_push> is depricated in favor of B<print> and will be removed in
-the 0.7 release.
-
 =cut
 
-sub body_push {
-    my $self = shift;
-
-    $self->print(@_);
-}
-
 sub print {
     my $self = shift;
 
@@ -545,11 +610,18 @@ or site configuration.  Multiple `/'s are collapsed.  This function will
 not be able to return theme dependent URLs except during the actual rendering
 of a page.
 
+If the url built from B<array> begins with `/', then the link is absolute with
+respect to the top of the site.  Otherwise, it is relative to the page being
+produced.
+
 =cut
+# ' for Emacs
+# ` for Emacs
+
 
 sub url {
     my $self = shift;
-    my $url = join("", $self->{'URLBASES'}->{'URLBASE'}, @_);
+    my $url = join("", @_);
 
     my(@subs) = ($url =~ m/\@\@(.*?)\@\@/g);
 
@@ -560,7 +632,12 @@ sub url {
 	    $url =~ s{\@\@$_\@\@}{$self->{'URLBASES'}->{$_}}g;
 	}
     }
-    $url =~ s!//!/!g;
+
+    if($url =~ /^\//) {
+	$url = $self->{'URLBASES'}->{'URLBASE'} . "/$url";
+    }
+
+    1 while($url =~ s!//!/!g);
 
     if($self->{'SSL_URLS'}) {
 	$url =~ s/:(NO)SSL//g;
@@ -587,6 +664,8 @@ will use Apache's URl->filename translation code if running under mod_perl.
 Otherwise, tacks the document root on the beginning.
 
 =cut
+# ` for Emacs
+# ' for Emacs
 
 sub filename {
     my $engine = shift;
@@ -608,6 +687,90 @@ sub filename {
 
 =pod
 
+=item $engine->smarttext(B<array>)
+
+This function will accept the B<array> of plain text and return a string
+of HTML formatted text.
+
+HTML character entities are preserved.  Also, &star; and &under; are
+translated to `*' and `_' respectively.  Paragraphs beginning with whitespace
+are quoted as preformatted text.  The ampersand, greater-than and less-than
+are translated to character entities.  Therefore, no HTML may be included in
+the input text.  Text may be _underlined_ or made *bold*.
+
+=cut
+
+# ' for Emacs
+# ` for Emacs
+
+# following is based on _The Perl Cookbook_
+
+sub _smarttext_render_p {
+    my($p) = shift;
+	
+    $p =~ s/%/%25/gs;
+    $p =~ s/&(.*?);/%%$1%%/gs;
+    $p =~ s/&/&amp;/gs;
+    $p =~ s/</&lt;/gs;
+    $p =~ s/>/&gt;/gs;
+    $p =~ s/%%(.*?)%%/&$1;/gs;
+    $p =~ s/%25/%/gs;
+    
+    $p =~ tr/\n/ /;
+    $p =~ s{^(&gt;.*)}  {$1<br>}gms;        # quoted text
+    $p =~   s{&lt;URL:\s*(.*?)&gt;}   {<a href="$1">$1</a>}gsi 
+	||   # these are for URLs
+	    s{((http|ftp|https):\S+)} {<a href="$1">$1</a>}gs; 
+    $p =~ s{\*(.*?)\*} {<strong>$1</strong>}gs;
+    $p =~ s{\b_(.*?)\_\b} {<i>$1</i>}gs;
+    $p =~ s{&star;}            {*}gsi;
+    $p =~ s{&under;}           {_}gsi;
+    $p = "<p>$p</p>";
+    
+    return $p;
+}
+
+sub smarttext {
+    my $engine = shift;
+    chomp(@_);
+    my $text = join(" \n", @_);
+    my (@text) = split(/\n/, $text);
+
+    my $paragraph;
+    my $output;
+    my $line;
+    my $p;
+
+    while(@text) {
+	$line = shift @text;
+	$line =~ s/\r//gs;
+	if($line =~ /^\s*$/ && $p) {
+	    $output .= &_smarttext_render_p($p);
+	    $p = '';
+	} elsif($line =~ /^\s/) {
+	    $line =~ s{(.*)$ } {<pre>\n$1</pre>\n}sx;
+	    if($p) {
+		$output .= &_smarttext_render_p($p);
+		$p = '';
+	    }
+	    $output .= $line; 
+	} else {
+	    $p .= "$line";
+	}	
+    }
+
+    # do last paragraph...
+    $output .= &_smarttext_render_p($p);
+
+    if(wantarray) {
+	return split(/\n/, $output);
+    } else {
+	return $output;
+    }
+}
+
+=pod
+
 =item $engine->render_content
 
 This function is the main workhorse returning an array resulting from rendering
@@ -703,22 +866,24 @@ sub render_page {
     my(@output, @head);
     my($theme, $layout, $css);
 
+    my $version = $CGI::WeT::VERSION || $CGI::WeT::Engine::VERSION;
+
     push(@head,
          map("<meta name=\"$_\" content=\"" . scalar($self->header($_)) 
-	     . "\">",
+	     . "\">\n",
              grep(defined $$self{HEADERS}->{$_},
                   'Author', 'Keywords', 'Date'
                   )
              )
          );
-    push(@head, "<meta name=\"Generator\" content=\"CGI::WeT $CGI::WeT::Engine::VERSION\">");
+    push(@head, "<meta name=\"Generator\" content=\"CGI::WeT $version\">\n");
     push(@head, "<meta name=\"Theme\" content=\"", 
 	 $self->argument('theme'),
-	 "\">");
+	 "\">\n");
 
     push(@head, "<title>", "$$self{SITENAME} - ", 
 	 scalar($self->header('Title')),
-         "</title>");
+         "</title>\n");
 
     $self->{'THEME'} ||= new CGI::WeT::Theme($self->argument('theme'))
 	|| new CGI::WeT::Theme($self->{'DEFAULT_THEME'});
@@ -760,25 +925,38 @@ sub render_page {
 	}
     }
     
-    return ("<html>", "<head>", @head, "</head>", @output, "</html>");
+    return ("<html>\n", "<head>\n", @head, "\n</head>\n", @output, 
+	    "\n</html>\n");
 }
 
 sub handler {
-    my $r = shift;
-    my $filename = $r->filename;
-    my ($key, $val);
-
-    {
-	no strict;
-	return Apache::Constants::NOT_FOUND
-	    unless open IN, "<$filename";
+    my $engine = new CGI::WeT::Engine;
+    my($r, $fh, $res);
+    
+    if($engine->{'FILTERED'}) {
+	$r = $_[0];
+	($fh, $res) = Apache->filter_input();
+    } else {
+	$r = shift;
+	$fh = new IO::File;
+	my $filename = $r->filename;
+	if($fh->open("<$filename")) {
+	    $res = Apache::Constants::OK;
+	} else {
+	    $res = Apache::Constants::NOT_FOUND;
+	}
     }
 
-    my $engine = new CGI::WeT::Engine;
+    unless($res == Apache::Constants::OK) {
+	$engine->internal_use_only(); # don't output the html
+	return $res; 
+    }
 
-    while(<IN>) {
+    my ($key, $val);
+
+    while(<$fh>) {
 	last if /^\s*$/;
-	next if /^\s*#/;
+	next if /^\s*\#/;
 	chomp;
 	if(/^[A-Za-z_]+:/) {
 	    ($key, $val) = split(/:\s*/,$_,2);
@@ -787,25 +965,27 @@ sub handler {
 	}
 	$engine->headers_push($key, $val);
     }
+
+    _handler($fh, $engine);
+    
+    return Apache::Constants::OK unless $r->is_initial_req;
+    return Apache::Constants::DONE;
+    # rendering done on $engine destruction
+}
+
+#
+# _handler is here to handle any translations to be done on the contents
+# of the html...  nothing too great so far
+#
+sub _handler {
+    my($in, $out) = @_;
     
     #
     # slurp up the rest of the file
     #
-    $engine->body_push((<IN>));
-    close IN;
-    
-    $r->content_type('text/html');
-
-    $r->send_http_header;
-
-    $r->print( $engine->render_page );
-
-    {
-	no strict;
-	return Apache::Constants::OK;
-    }
+    $out->print($_) while(<$in>);
 }
-
+    
 1;
 
 =pod
@@ -817,11 +997,11 @@ CGI::WeT(3),
 CGI::WeT::Theme(3),
 CGI::WeT::Modules(3),
 
-CGI::WeT notes at C<http://people.physics.tamu.edu/jsmith/wet-perls/>
+CGI::WeT notes at C<http://hex.nostrum.com/cgi-wet/>
 
 =head1 AUTHORS
 
  Written by James G. Smith.
- Copyright (C) 1999.  Released under the GNU General Public License v. 2.
+ Copyright (C) 1999.  Released under the Artistic License.
 
 =cut
